# _*_ coding: iso-8859-1 _*_

import servicemanager
import win32com.client
import win32com.server.policy
import pythoncom
import win32security
import win32process
import win32profile
import win32con
import win32event
import win32ts


## from Sens.h
SENSGUID_PUBLISHER = "{5fee1bd6-5b9b-11d1-8dd2-00aa004abd5e}"
SENSGUID_EVENTCLASS_LOGON = "{d5978630-5b9f-11d1-8dd2-00aa004abd5e}"

## from EventSys.h
PROGID_EventSystem = "EventSystem.EventSystem"
PROGID_EventSubscription = "EventSystem.EventSubscription"

IID_ISensLogon = "{d597bab3-5b9f-11d1-8dd2-00aa004abd5e}"


class SensLogon(win32com.server.policy.DesignatedWrapPolicy):
    _com_interfaces_=[IID_ISensLogon]
    _public_methods_=[
        'Logon',
        'Logoff',
        'StartShell',
        'DisplayLock',
        'DisplayUnlock',
        'StartScreenSaver',
        'StopScreenSaver'
        ]

    def __init__(self):
        self._wrap_(self)

    def Logon(self, *args):
        logevent('Logon : %s'%[args])
        logon(args)
 
    def Logoff(self, *args):
        logevent('Logoff : %s'%[args])

    def StartShell(self, *args):
        logevent('StartShell : %s'%[args])
    
    def DisplayLock(self, *args):
        logevent('DisplayLock : %s'%[args])
    
    def DisplayUnlock(self, *args):
        logevent('DisplayUnlock : %s'%[args])
        # for easier tests
        logon(args)
    
    def StartScreenSaver(self, *args):
        logevent('StartScreenSaver : %s'%[args])
    
    def StopScreenSaver(self, *args):
        logevent('StopScreenSaver : %s'%[args])


def logevent(msg, evtid=0xF000):
    """log into windows event manager
    """
    servicemanager.LogMsg(
            servicemanager.EVENTLOG_INFORMATION_TYPE,
            evtid, #  generic message
            (msg, '')
            )

def register():
    logevent('Registring ISensLogon')

    sl=SensLogon()
    subscription_interface=pythoncom.WrapObject(sl)
    
    event_system=win32com.client.Dispatch(PROGID_EventSystem)
    
    event_subscription=win32com.client.Dispatch(PROGID_EventSubscription)
    event_subscription.EventClassID=SENSGUID_EVENTCLASS_LOGON
    event_subscription.PublisherID=SENSGUID_PUBLISHER
    event_subscription.SubscriptionName='Python subscription'
    event_subscription.SubscriberInterface=subscription_interface
    
    event_system.Store(PROGID_EventSubscription, event_subscription)
    
    pythoncom.PumpMessages()
    logevent('ISensLogon stopped')


def logon(args):
    domain, username = args[0].split('\\')
    sess_id = get_sess_id(username)
    user_token = win32ts.WTSQueryUserToken(int(sess_id))
    execute('notepad.exe')
    execute_as_user(user_token, 'notepad.exe')

def get_sess_id(username):
    for sn in win32security.LsaEnumerateLogonSessions():
        sn_info = win32security.LsaGetLogonSessionData(sn)
        if sn_info['UserName'].lower() == username.lower():
            return sn_info['Session']

def execute(cmd, hide=False, nowait=True, waitinput=False):
    appName = None
    commandLine = cmd
    processAttributes = None
    threadAttributes = None
    inheritHandles = False
    creationFlags = win32con.CREATE_DEFAULT_ERROR_MODE
    newEnvironnement = None
    currentDirectory = None
    startupinfo = win32process.STARTUPINFO()
    if hide:
        startupinfo.dwFlags = win32process.STARTF_USESHOWWINDOW
        startupinfo.wShowWindow = win32con.SW_HIDE
    handle, hthread, process_id, thread_id = win32process.CreateProcess(appName,
                                                                       commandLine,
                                                                       processAttributes,
                                                                       threadAttributes,
                                                                       inheritHandles,
                                                                       creationFlags,
                                                                       newEnvironnement,
                                                                       currentDirectory,
                                                                       startupinfo)
     
    if waitinput:
        # wait program initialisation
        win32event.WaitForInputIdle(handle, win32event.INFINITE)
    elif nowait: return 0
    # wait program exits
    win32event.WaitForSingleObject(handle, win32event.INFINITE)
    return int(win32process.GetExitCodeProcess(handle))

def execute_as_user(usertoken, cmd, hide=False, nowait=True, waitinput=False):
    appName = None
    commandLine = cmd
    processAttributes = None
    threadAttributes = None
    inheritHandles = False
    creationFlags = win32con.CREATE_DEFAULT_ERROR_MODE
    newEnvironnement = win32profile.CreateEnvironmentBlock(usertoken, False)
    currentDirectory = None
    startupinfo = win32process.STARTUPINFO()
    if hide:
        startupinfo.dwFlags = win32process.STARTF_USESHOWWINDOW
        startupinfo.wShowWindow = win32con.SW_HIDE
    handle, hthread, process_id, thread_id = win32process.CreateProcessAsUser(usertoken,
                                                                       appName,
                                                                       commandLine,
                                                                       processAttributes,
                                                                       threadAttributes,
                                                                       inheritHandles,
                                                                       creationFlags,
                                                                       newEnvironnement,
                                                                       currentDirectory,
                                                                       startupinfo)
     
    if waitinput:
        # wait program initialisation
        win32event.WaitForInputIdle(handle, win32event.INFINITE)
    elif nowait: return 0
    # wait program exits
    win32event.WaitForSingleObject(handle, win32event.INFINITE)
    return int(win32process.GetExitCodeProcess(handle))
